@@ -1,15 +1,16 @@
 # Get the management API data
 #' @include get-data.R
+#' @include convert.R
 #'
 get_mgmt <- function(path, query, cols, token, verbose = getOption("rga.verbose", FALSE)) {
     data_json <- get_data(type = "mgmt", path = path, query = query, token = token, verbose = verbose)
-    if (data_json$totalResults > 0 && !is.null(data_json[["items"]])) {
-        data_df <- data_json[["items"]]
-        data_df <- data_df[, names(data_df) %in% cols]
-    } else {
-        data_df <- data.frame(matrix(NA, nrow = 1L, ncol = length(cols)))
-        colnames(data_df) <- cols
+    items <- data_json[["items"]]
+    if (data_json$totalResults == 0 || is.null(items)) {
+        if (verbose)
+            message("No results were obtained.")
+        items <- data.frame(matrix(NA, nrow = 1L, ncol = length(cols)))
     }
+    data_df <- build_df(type = "mgmt", items, cols, verbose = verbose)
     return(data_df)
 }
 
@@ -38,9 +39,9 @@ get_mgmt <- function(path, query, cols, token, verbose = getOption("rga.verbose"
 get_accounts = function(start.index = NULL, max.results = NULL, token, verbose = getOption("rga.verbose", FALSE)) {
     path <- "accounts"
     query <- list(start.index = start.index, max.results = max.results)
-    data_df <- get_mgmt(path = path, query = query, token = token, verbose = verbose,
-                        cols = c("id", "name", "created", "updated"))
-    return(data_df)
+    res <- get_mgmt(path = path, query = query, token = token, verbose = verbose,
+                    cols = c("id", "name", "created", "updated"))
+    return(res)
 }
 
 #' @title Lists web properties which the user has access to
@@ -74,9 +75,9 @@ get_accounts = function(start.index = NULL, max.results = NULL, token, verbose =
 get_webproperties = function(account.id = "~all", start.index = NULL, max.results = NULL, token, verbose = getOption("rga.verbose", FALSE)) {
     path <- paste("accounts", account.id, "webproperties", sep = "/")
     query <- list(start.index = start.index, max.results = max.results)
-    data_df <- get_mgmt(path = path, query = query, token = token, verbose = verbose,
-                        cols = c("accountId", "id", "name", "websiteUrl", "level", "profileCount", "industryVertical", "created", "updated"))
-    return(data_df)
+    res <- get_mgmt(path = path, query = query, token = token, verbose = verbose,
+                    cols = c("accountId", "id", "name", "websiteUrl", "level", "profileCount", "industryVertical", "created", "updated"))
+    return(res)
 }
 
 #' @title Lists views (profiles) which the user has access to
@@ -115,9 +116,9 @@ get_webproperties = function(account.id = "~all", start.index = NULL, max.result
 get_profiles = function(account.id = "~all", webproperty.id = "~all", start.index = NULL, max.results = NULL, token, verbose = getOption("rga.verbose", FALSE)) {
     path <- paste("accounts", account.id, "webproperties", webproperty.id, "profiles", sep = "/")
     query <- list(start.index = start.index, max.results = max.results)
-    data_df <- get_mgmt(path = path, query = query, token = token, verbose = verbose,
-                        cols = c("accountId", "webPropertyId", "id", "name", "websiteUrl", "type", "siteSearchQueryParameters", "siteSearchCategoryParameters", "eCommerceTracking", "currency", "timezone", "created", "updated"))
-    return(data_df)
+    res <- get_mgmt(path = path, query = query, token = token, verbose = verbose,
+                    cols = c("accountId", "webPropertyId", "id", "name", "websiteUrl", "type", "siteSearchQueryParameters", "siteSearchCategoryParameters", "eCommerceTracking", "currency", "timezone", "created", "updated"))
+    return(res)
 }
 
 #' @title Lists  Lists goals which the user has access to
@@ -154,9 +155,9 @@ get_profiles = function(account.id = "~all", webproperty.id = "~all", start.inde
 get_goals = function(account.id = "~all", webproperty.id = "~all", profile.id = "~all", start.index = NULL, max.results = NULL, token, verbose = getOption("rga.verbose", FALSE)) {
     path <- paste("accounts", account.id, "webproperties", webproperty.id, "profiles", profile.id, "goals", sep = "/")
     query <- list(start.index = start.index, max.results = max.results)
-    data_df <- get_mgmt(path = path, query = query, token = token, verbose = verbose,
-                        cols = c("accountId", "webPropertyId", "profileId", "id", "name", "value", "active", "type", "created", "updated"))
-    return(data_df)
+    res <- get_mgmt(path = path, query = query, token = token, verbose = verbose,
+                    cols = c("accountId", "webPropertyId", "profileId", "id", "name", "value", "active", "type", "created", "updated"))
+    return(res)
 }
 
 #' @title Lists segments which the user has access to
@@ -189,7 +190,7 @@ get_goals = function(account.id = "~all", webproperty.id = "~all", profile.id =
 get_segments = function(start.index = NULL, max.results = NULL, token, verbose = getOption("rga.verbose", FALSE)) {
     path <- "segments"
     query <- list(start.index = start.index, max.results = max.results)
-    data_df <- get_mgmt(path = path, query = query, token = token, verbose = verbose,
-                        cols = c("segmentId", "id", "name", "definition", "type", "created", "updated"))
-    return(data_df)
+    res <- get_mgmt(path = path, query = query, token = token, verbose = verbose,
+                    cols = c("segmentId", "id", "name", "definition", "type", "created", "updated"))
+    return(res)
 }
